@@ -47,9 +47,11 @@ static int test_attach_write_file(void* payload)
 	if (f) {
 		printf("yes\n");
 		fclose(f);
+		fflush(NULL);
 		return 1;
 	}
 	printf("no\n");
+	fflush(NULL);
 	return 0;
 }
 
@@ -69,7 +71,7 @@ static int do_test_file_open(struct lxc_container *c, char *fnam)
 
 	ret = pipe(pipefd);
 	if (ret < 0) {
-		fprintf(stderr, "pipe failed %d", ret);
+		fprintf(stderr, "pipe failed %d\n", ret);
 		return fret;
 	}
 	attach_options.stdout_fd = pipefd[1];
@@ -77,13 +79,13 @@ static int do_test_file_open(struct lxc_container *c, char *fnam)
 	attach_options.attach_flags |= LXC_ATTACH_LSM_NOW;
 	ret = c->attach(c, test_attach_write_file, fnam, &attach_options, &pid);
 	if (ret < 0) {
-		fprintf(stderr, "attach failed");
+		fprintf(stderr, "attach failed\n");
 		goto err1;
 	}
 
 	ret = read(pipefd[0], result, sizeof(result)-1);
 	if (ret < 0) {
-		fprintf(stderr, "read failed %d", ret);
+		fprintf(stderr, "read failed %d\n", ret);
 		goto err2;
 	}
 
@@ -117,7 +119,7 @@ static bool test_aa_policy(struct lxc_container *c)
 	for (i = 0; files_to_deny[i]; i++) {
 		ret = do_test_file_open(c, files_to_deny[i]);
 		if (ret < 0) {
-			fprintf(stderr, "attach failed; skipping test");
+			fprintf(stderr, "attach failed; skipping test\n");
 			return true;
 		}
 		if (ret > 0) {
@@ -131,7 +133,7 @@ static bool test_aa_policy(struct lxc_container *c)
 	for (i = 0; files_to_allow[i]; i++) {
 		ret = do_test_file_open(c, files_to_allow[i]);
 		if (ret < 0) {
-			fprintf(stderr, "attach failed; skipping test");
+			fprintf(stderr, "attach failed; skipping test\n");
 			return true;
 		}
 		if (ret == 0) {
@@ -164,13 +166,19 @@ int main(int argc, char *argv[])
 		goto err;
 	}
 	c->save_config(c, NULL);
-	if (!c->createl(c, "download", NULL, NULL, 0, "-d", "ubuntu", "-r", "trusty", "-a", "amd64", NULL)) {
+	if (!c->createl(c, "busybox", NULL, NULL, 0, NULL)) {
 		fprintf(stderr, "%s: %d: failed to create container\n", __FILE__, __LINE__);
 		goto err;
 	}
+
+	c->clear_config_item(c, "lxc.mount.auto");
+	c->set_config_item(c, "lxc.mount.entry", "proc proc proc");
+	c->set_config_item(c, "lxc.mount.entry", "sysfs sys sysfs");
+	c->save_config(c, NULL);
+
 	c->want_daemonize(c, true);
 	if (!c->startl(c, 0, NULL)) {
-		fprintf(stderr, "Error starting container");
+		fprintf(stderr, "Error starting container\n");
 		goto err;
 	}
 
