@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
-#include "../lxc/lxccontainer.h"
+#include <lxc/lxccontainer.h>
 
 #include <unistd.h>
 #include <signal.h>
@@ -51,8 +51,8 @@ int main(int argc, char *argv[])
 	}
 	c->set_config_item(c, "lxc.network.link", "lxcbr0");
 	c->set_config_item(c, "lxc.network.flags", "up");
-	if (!c->createl(c, "ubuntu", "-r", "lucid", NULL)) {
-		fprintf(stderr, "%d: failed to create a lucid container\n", __LINE__);
+	if (!c->createl(c, "busybox", NULL, NULL, 0, NULL)) {
+		fprintf(stderr, "%d: failed to create a container\n", __LINE__);
 		goto out;
 	}
 
@@ -61,17 +61,22 @@ int main(int argc, char *argv[])
 		goto out;
 	}
 
+	c->clear_config(c);
 	c->load_config(c, NULL);
-	c->want_daemonize(c);
+	c->want_daemonize(c, true);
 	if (!c->startl(c, 0, NULL)) {
 		fprintf(stderr, "%d: failed to start %s\n", __LINE__, MYNAME);
 		goto out;
 	}
-	fprintf(stderr, "%d: %s started, you have 60 seconds to test a console\n", __LINE__, MYNAME);
-	sleep(60);  // wait a minute to let user connect to console
 
-	if (!c->shutdown(c, 60)) {
+	/* Wait for init to be ready for SIGPWR */
+	sleep(20);
+
+	if (!c->shutdown(c, 120)) {
 		fprintf(stderr, "%d: failed to shut down %s\n", __LINE__, MYNAME);
+		if (!c->stop(c)) {
+			fprintf(stderr, "%d: failed to kill %s\n", __LINE__, MYNAME);
+		}
 		goto out;
 	}
 
@@ -88,6 +93,10 @@ int main(int argc, char *argv[])
 	fprintf(stderr, "all lxc_container tests passed for %s\n", c->name);
 	ret = 0;
 out:
+	if (c && c->is_defined(c)) {
+		c->destroy(c);
+	}
+
 	lxc_container_put(c);
 	exit(ret);
 }
